<?xml version="1.0"?>

<st-source>
<!-- Changes in change set PV-Datachannel-diffs-Green.289-286 -->
<time-stamp>From VisualWorks®, 7.3.1 of April 20, 2005 on October 14, 2008 at 12:17:17 pm</time-stamp>


<methods>
<class-id>PROVISO.PvFmCommandLog</class-id> <category>private-actions</category>

<body package="PV-FM-Core" selector="checkFileSizeOf:and:ifError:ifUnsupported:ifZeroLength:">checkFileSizeOf: srcURIOrString
	and: destURIOrString
	ifError: errorBlock
	ifUnsupported: unsupportedBlock
	ifZeroLength: zeroLengthBlock
	"&lt;Boolean&gt; Answer whether the file sizes of both URIs match"
	
	| destFileSize srcFileSize |
	destFileSize := 
		self 
			getFileFizeOf: destURIOrString
			ifError: [^errorBlock value]
			ifUnsupported: [^unsupportedBlock value].
	srcFileSize := 
		self 
			getFileFizeOf: srcURIOrString
			ifError: [^errorBlock value]
			ifUnsupported: [^unsupportedBlock value].
	^srcFileSize = destFileSize 
		ifTrue: 
			[srcFileSize isZero 
				ifTrue: [^zeroLengthBlock value].
			true]
		ifFalse: 
			[self 
				logMismatchedFileSizes: srcURIOrString asURI -&gt; srcFileSize
				and: destURIOrString asURI -&gt; destFileSize.
			false]</body>

<body package="PV-FM-Core" selector="getFileFizeOf:ifError:ifUnsupported:">getFileFizeOf: aURIOrString ifError: errorBlock ifUnsupported: unsupportedBlock
	"&lt;Boolean&gt; Answer whether the file size is zero"

	| fileSize uri |
	uri := aURIOrString asURI.
	uri canDetermineFileSize ifFalse: [^unsupportedBlock value].
	
	[fileSize := uri fileSize]
		on: self errorsToHandle
		do: 
			[ :ex |
			self logUnableToRetrieveFileSize: uri asURI.
			^errorBlock value].
	^fileSize</body>
</methods>


<methods>
<class-id>PROVISO.PvFmCommandLog</class-id> <category>private-actions</category>

<body package="PV-FM-Core" selector="doCopyFromAnyOf:to:copySelector:retryLimit:">doCopyFromAnyOf: sourceURIs to: destURI copySelector: copySelector retryLimit: retryLimit
	"&lt;Boolean&gt; Attempt to copy any one of the given source URIs to the given destURI"

	| retryCount retryLimitNumber decodedSrcURIs isZeroLength |
	retryLimitNumber := retryLimit asNumber.
	retryCount := 1.
	[retryCount &lt;= retryLimitNumber] whileTrue:
		[isZeroLength := false.
		 (decodedSrcURIs := self decodeURIList: sourceURIs) do:
			[ :ea | 
			(((self doCopyFrom: ea to: destURI copySelector: copySelector) ~~ false) and:
				[self checkFileSizeOf: ea and: destURI ifError: [false] ifUnsupported: [true] ifZeroLength: [isZeroLength := true. false]])
					ifTrue: [^true]].
		(self timeFacade delayForMilliseconds: 200) wait.
		retryCount := retryCount + 1].
	"if file stayed zero length for 5 retries then it must be zero length, let it go"
	isZeroLength ifTrue: [^true].
	"We've tried all sourceURIs up to a retryLimit number of times, checking file sizes and we still can't acquire the resource...log the failure and raise an exception."
	self logRetryLimitExceeded: decodedSrcURIs.
	self error: 'failed to complete #copyFromAnyOf:to:copySelector:retryLimit: command'.
	^false</body>
</methods>


<remove-selector>
<class-id>PROVISO.PvFmCommandLog</class-id> <selector>checkFileSizeOf:and:ifError:ifUnsupported:</selector>
</remove-selector>

<methods>
<class-id>PROVISO.UBA.SAMPassThruSchema</class-id> <category>acquisition</category>

<body package="PVAdaptor-Schema" selector="inputsToAcquire">inputsToAcquire
	
	| all |
	all := super inputsToAcquire.
	^self lastExportTimestamp 
		ifNil: [all]
		ifNotNil: 
			[:lastExport | all select: [:each | each timestamp asPvTimestamp &lt; lastExport]]</body>
</methods>


<methods>
<class-id>PROVISO.PvSAXParser</class-id> <category>actions</category>

<body package="PV-Support" selector="parseElement">parseElement
	
	self atEnd 
		ifTrue: 
			[sax endDocumentFragment.
			self closeAllFiles.
			^nil].
	atEndOfRecord := false.
	[hereChar == $&lt;] 
		whileFalse: 
			[self getNextChar.
			hereChar ifNil: [^nil]].
	self getElement.
	^self currentRecord</body>
</methods>


<methods>
<class-id>PROVISO.ForeignDomainMethod</class-id> <category>private</category>

<body package="SAM-DomainModel" selector="copy:to:">copy: aURI to: localURI 
	
	| delay copyAttempts |
	delay := 500.
	copyAttempts := 0.
	[(copyAttempts := copyAttempts + 1) &gt; 5 
		ifTrue: 
			[^aURI fileSize isZero 
				ifTrue: 
					[self system 
						logDebugFour: #ACQUIRE_FILE
						text: 'Acquired zero length ' , localURI printString]
				ifFalse: [self error: 'copy attempt limit reached']].
	aURI copyTo: localURI.
	localURI fileSize = aURI fileSize and: [aURI fileSize isZero not]] 
		whileFalse: [(self timeFacade delayForMilliseconds: (delay := delay * 2)) wait]</body>

<body package="SAM-DomainModel" selector="waitFor:">waitFor: aURI 
	
	"Wait up to 5 minutes for an export file to appear"
	| delay deadline |
	deadline := Time totalSeconds + 300.
	delay := 500.
	[aURI exists] 
		whileFalse: 
			[Time totalSeconds &gt; deadline 
				ifTrue: [self error: 'export file not found'].
			(self timeFacade delayForMilliseconds: (delay := delay * 2)) wait]</body>
</methods>


<methods>
<class-id>PROVISO.ForeignDomainMethod</class-id> <category>invocation</category>

<body package="SAM-DomainModel" selector="acquireFile:">acquireFile: aURI 
	
	| localURI |
	self system 
		logDebugFour: #ACQUIRE_FILE
		text: 'Acquiring file ' , aURI asEncryptedString.
	localURI := self exportFilenameLocalDirectory , aURI tail.
	self waitFor: aURI.
	self 
		copy: aURI
		to: localURI.
	self system 
		logDebugFour: #ACQUIRE_FILE
		text: 'Successfully acquired file into ' , localURI printString.
	^localURI</body>
</methods>



</st-source>
